#include "UART.h"
#include "USBHost.h"

USBHost USB;
UART Serial;

void test_enumerate();
void hexDump(uint8_t * buffer, int length);

int main()
{
    USB.start();
    Serial.start(31, 30, 0, 115200);

    while (1) {
        test_enumerate();
        waitcnt(CNT + CLKFREQ);
    }

    return 0;
}

void test_enumerate()
{
    int rc;

    if ((rc = USB.enumerate()) != 0) {
        Serial.print("Can't enumerate device (");
        Serial.print(rc);
        Serial.println(")");
        return;
    }

    Serial.print("\014");
    Serial.print("Found device ");
    Serial.print(USB.vendor_id(), HEX, 4);
    Serial.print(":");
    Serial.println(USB.product_id(), HEX, 4);

    Serial.println();
    Serial.println("Raw Device Descriptor");
    hexDump(USB.alloc_buf_dd, DEVDESC_LEN);

    Serial.println();
    Serial.println("Device Configuration");

    uint8_t * ifd = USB.first_interface();
    while (ifd != NULL) {
        Serial.print("Interface ptr=");
        Serial.print((int)ifd, HEX, 4);
        Serial.print(" number=");
        Serial.print(ifd[IFDESC_bInterfaceNumber], HEX, 2);
        Serial.print(" alt=");
        Serial.print(ifd[IFDESC_bAlternateSetting], HEX, 2);
        Serial.print(" class=");
        Serial.print(ifd[IFDESC_bInterfaceClass], HEX, 2);
        Serial.print(" subclass=");
        Serial.println(ifd[IFDESC_bInterfaceSubClass], HEX, 2);

        uint8_t * epd = ifd;
        while ((epd = USB.next_endpoint(epd)) != NULL) {
            Serial.print("    Endpoint ptr=");
            Serial.print((int)epd, HEX, 4);
            Serial.print(" address=");
            Serial.print(epd[EPDESC_bEndpointAddress], HEX, 2);
            Serial.print(" maxpacket=");
            Serial.println(epd[EPDESC_wMaxPacketSize] | (epd[EPDESC_wMaxPacketSize + 1] << 8), HEX, 4);
        }

        ifd = USB.next_interface(ifd);
    }

    if ((rc = USB.configure()) != 0) {
        Serial.print("Error configuring device (");
        Serial.print(rc);
        Serial.println(")");
    }
}

void hexDump(uint8_t * buffer, int length)
{
    for (int i = 0; i < length; i++, buffer++) {
        Serial.print(*buffer, HEX, 2);
        Serial.print(" ");
    }
    Serial.println();
}
