#include "UART.h"
#include "USBHost.h"

USBHost USB;
UART Serial;

void test_hid();
void hex_dump(uint8_t * buffer, int length);

uint8_t usb_buf[64];

int main()
{
    USB.start();
    Serial.start(31, 30, 0, 115200);

    while (1) {
        test_hid();
        waitcnt(CNT + CLKFREQ);
    }

    return 0;
}

void test_hid()
{
    int rc;
    uint8_t * ifd, * epd;

    if ((rc = USB.enumerate()) != 0) {
        Serial.print("Can't enumerate device (");
        Serial.print(rc);
        Serial.println(")");
        return;
    }

    Serial.print("\014");
    Serial.print("Found device ");
    Serial.print(USB.vendor_id(), HEX, 4);
    Serial.print(":");
    Serial.println(USB.product_id(), HEX, 4);

    if ((rc = USB.configure()) != 0) {
        Serial.print("Error configuring device (");
        Serial.print(rc);
        Serial.println(")");
        return;
    }

    if ((ifd = USB.find_interface(3)) == NULL) {
        Serial.println("Device has no HID interface");
        return;
    }
    epd = USB.next_endpoint(ifd);

    while(USB.get_port_connection() != PORTC_NO_DEVICE) {
        rc = USB.interrupt_read(epd, usb_buf, 64);
        if (rc > 0) {
            Serial.print("[");
            Serial.print(rc);
            Serial.print(" bytes] ");
            hex_dump(usb_buf, rc);
        }
        waitcnt(CNT + CLKFREQ / 10);
    }
}

void hex_dump(uint8_t * buffer, int length)
{
    for (int i = 0; i < length; i++, buffer++) {
        Serial.print(*buffer, HEX, 2);
        Serial.print(" ");
    }
    Serial.println();
}
